import React, { useState, useEffect, useRef, useMemo } from 'react';
import {
  scaleTime,
  extent,
  scaleLinear,
  max,
  select,
  line,
  timeFormat,
  brushX,
  event
} from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';

const xValue = (d) => d.date;
const yValue = (d) => d.death;

const margin = {
  top: 40,
  right: 80,
  bottom: 80,
  left: 150,
};

const formatDate = timeFormat('%b %d');

export const LineChart = ({ data, width, height }) => {
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = useMemo(() => scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth]), [data, xValue, innerWidth]);

  const yScale = useMemo(() => scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0]), [data, yValue, innerHeight]);

  const lineGenerator = useMemo(() => line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d))), [xScale, yScale, yValue, xValue]);
  
  
	const [brushed, setBrushed] = useState(null);

  const brushRef = useRef();
  useEffect(() => {
    const brush = brushX().extent([[0, 0], [innerWidth, innerHeight]]);
    brush(select(brushRef.current));
    brush.on('brush end', () => {
      setBrushed(event.selection && event.selection.map(xScale.invert));
    });
  }, [innerWidth, innerHeight]);
  
  
  const filteredData = brushed ? data.filter(d => xValue(d) >= brushed[0] && xValue(d) <= brushed[1]) : data

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        <path d={lineGenerator(data)} />
        <g ref={brushRef}/>
      </g>
    </svg>
  );
};
